{***********************************************************************}
{               Components Collection                                   }
{                                                                       }
{               Copyright (C) 1996 by Artem A. Berman                   }
{               E-mail: art@aber.kherson.ua                             }
{***********************************************************************}
{ This components can be freely used and distributed in commercial and  }
{ private environments, provied this notice is not modified in any way. }
{ ----------------------------------------------------------------------}
{ Version 1.01. Date last modified:  09/27/96                           }
{ ----------------------------------------------------------------------}

unit Appexec;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Controls,  Forms, CloseApp;

type
  TShowType = (S_HIDE, S_MINIMIZE, S_RESTORE, S_SHOW, S_SHOWMAXIMIZED,
               S_SHOWMINIMIZED, S_SHOWMINNOACTIVE, S_SHOWNA, S_SHOWNOACTIVATE,
               S_SHOWNORMAL);

  TAppExecute = class(TComponent)
  private
    fAppName,
    fAppParam: string;
    fWaitForEnd: Boolean;
    fShowType: TShowType;
    fAppInstance: Integer;
    fOnAppExec,
    fOnClose: TNotifyEvent;
  public
    procedure Execute;
    procedure Close;
    constructor Create(aOwner: TComponent); override;
  published
    property AppName: string read fAppName write fAppName;
    property AppParam: string read fAppParam write fAppParam;
    property WaitForEnd: Boolean read fWaitForEnd write fWaitForEnd default False;
    property ShowType: TShowType read fShowType write fShowType;
    property AppInstance: Integer read fAppInstance;
    property OnExecute: TNotifyEvent read fOnAppExec write fOnAppExec;
    property OnClose: TNotifyEvent read fOnClose write fOnClose;
  end;

procedure Register;

implementation

constructor TAppExecute.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  fAppInstance := 0;
  fShowType := S_SHOW;
end;


procedure TAppExecute.Execute;
var
  Buff: array [0..80] of Char;
  Cmd: string;
  Res, fShow: Integer;
begin
  Cmd := fAppName + ' ' + fAppParam;
  StrPCopy(Buff, Cmd);

  case fShowType of
    S_HIDE              : fShow := SW_HIDE;
    S_MINIMIZE          : fShow := SW_MINIMIZE;
    S_RESTORE           : fShow := SW_RESTORE;
    S_SHOW              : fShow := SW_SHOW;
    S_SHOWMAXIMIZED     : fShow := SW_SHOWMAXIMIZED;
    S_SHOWMINIMIZED     : fShow := SW_SHOWMINIMIZED;
    S_SHOWMINNOACTIVE   : fShow := SW_SHOWMINNOACTIVE;
    S_SHOWNA            : fShow := SW_SHOWNA;
    S_SHOWNOACTIVATE    : fShow := SW_SHOWNOACTIVATE;
    S_SHOWNORMAL        : fShow := SW_SHOWNORMAL;
  end;

  if Assigned(fOnAppExec) then fOnAppExec(Self);
  fAppInstance := WinExec(Buff, fShow);

  if fWaitForEnd then
  Repeat
    Res := GetModuleUsage(fAppInstance);
    Application.ProcessMessages;
  until Res = 0
end;

procedure TAppExecute.Close;
var
  Res: Integer;
begin
  Res := GetModuleUsage(fAppInstance);
  if Res <> 0 then
  begin
    if Assigned(fOnClose) then fOnClose(Self);
    CloseAppFromInst(fAppInstance);
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TAppExecute]);
end;

end.